@@ -4,10 +4,41 @@
 import xml.etree.ElementTree as ET
 
 from .enumerations import VersionBase, XMLNS, XSI
-from .utils import _StochasticDistributionType,  FileHeader, VersionBase, printToFile, ParameterAssignment
-from .exceptions import NotEnoughInputArguments, OpenSCENARIOVersionError
+from .utils import _StochasticDistributionType,  FileHeader, VersionBase, ParameterAssignment, Rule, convert_float
+from .exceptions import NotEnoughInputArguments, OpenSCENARIOVersionError, NotAValidElement
+from ..helpers import prettyprint, printToFile
 
 
+class _StochasticFactory():
+    @staticmethod
+    def parse_distribution(element):
+        if element.find('NormalDistribution') is not None:
+            return NormalDistribution.parse(element.find('NormalDistribution'))
+        elif element.find('UniformDistribution') is not None:
+            return UniformDistribution.parse(element.find('UniformDistribution'))
+        elif element.find('PoissonDistribution') is not None:
+            return PoissonDistribution.parse(element.find('PoissonDistribution'))
+        elif element.find('Histogram') is not None:
+            return Histogram.parse(element.find('Histogram'))
+        elif element.find('ProbabilityDistributionSet') is not None:
+            return ProbabilityDistributionSet.parse(element.find('ProbabilityDistributionSet'))
+        elif element.find('UserDefinedDistribution') is not None:
+            return NotImplementedError('UserDefinedDistribution is not implemented yet.')    
+        else:
+            raise NotAValidElement('element ', element ,'is not a valid Stochastic distribution')
+
+class _DeterministicFactory():
+    @staticmethod
+    def parse_distribution(element):
+        if element.find('DistributionSet') is not None:
+            return DistributionSet.parse(element.find('DistributionSet'))
+        elif element.find('DistributionRange') is not None:
+            return DistributionRange.parse(element.find('DistributionRange'))
+        elif element.find('UserDefinedDistribution') is not None:
+            return NotImplementedError('UserDefinedDistribution is not implemented yet.')
+        else:
+            raise NotAValidElement('element ', element ,'is not a valid deterministic distribution')
+
 class _HistogramBin():
     """ the _HistogramBin is used by Histogram to define the bins
         
@@ -25,6 +56,9 @@ class _HistogramBin():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -44,7 +78,7 @@ class _HistogramBin():
         """ 
         if range and not isinstance(range,Range):
             raise TypeError('range input is not of type Histogram')
-        self.weight = weight
+        self.weight = convert_float(weight)
         self.range = range
 
     def __eq__(self,other):
@@ -54,6 +88,27 @@ class _HistogramBin():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of _HistogramBin
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A _HistogramBin element (same as generated by the class itself)
+
+            Returns
+            -------
+                histogram (_HistogramBin): a _HistogramBin object
+                
+        """
+        range_element = element.find('Range')
+        if range_element is not None:
+            range = Range.parse(range_element)
+        else:
+            range = None
+
+        return _HistogramBin(convert_float(element.attrib['weight']), range)
+
     def get_attributes(self):
         """ returns the attributes of the HistogramBin as a dict
 
@@ -86,6 +141,9 @@ class _ProbabilityDistributionSetElement():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -104,7 +162,7 @@ class _ProbabilityDistributionSetElement():
 
         """ 
         self.value = value
-        self.weight = weight
+        self.weight = convert_float(weight)
 
     def __eq__(self,other):
         if isinstance(other,_ProbabilityDistributionSetElement):
@@ -112,6 +170,22 @@ class _ProbabilityDistributionSetElement():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of _ProbabilityDistributionSetElement
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A _ProbabilityDistributionSetElement element (same as generated by the class itself)
+
+            Returns
+            -------
+                pds (_ProbabilityDistributionSetElement): a _ProbabilityDistributionSetElement object
+                
+        """
+
+        return _ProbabilityDistributionSetElement(element.attrib['value'], convert_float(element.attrib['weight']))
+
     def get_attributes(self):
         """ returns the attributes of the _ProbabilityDistributionSetElement as a dict
 
@@ -143,6 +217,9 @@ class Range():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -160,8 +237,8 @@ class Range():
             upper (float): upper limit of the range
 
         """ 
-        self.lower = lower
-        self.upper = upper
+        self.lower = convert_float(lower)
+        self.upper = convert_float(upper)
 
     def __eq__(self,other):
         if isinstance(other,Range):
@@ -169,6 +246,22 @@ class Range():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of Range
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A Range element (same as generated by the class itself)
+
+            Returns
+            -------
+                range (Range): a Range object
+                
+        """
+
+        return Range(convert_float(element.attrib['lowerLimit']),convert_float(element.attrib['upperLimit']))
+
     def get_attributes(self):
         """ returns the attributes of the Range as a dict
 
@@ -184,9 +277,9 @@ class Range():
         return element
 
 
-## Stocastic distributions
-class Stocastic():
-    """ Creates the Stocastic type of parameter distributions
+## Stochastic distributions
+class Stochastic():
+    """ Creates the Stochastic type of parameter distributions
         
         Parameters
         ----------
@@ -205,6 +298,9 @@ class Stocastic():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             add_distribution(parametername, distribution)
                 Adds a parameter/distribution pair
 
@@ -216,7 +312,7 @@ class Stocastic():
 
     """
     def __init__(self,number_of_tests,random_seed=None):
-        """ initalizes the Stocastic 
+        """ initalizes the Stochastic 
 
         Parameters
         ----------
@@ -231,11 +327,37 @@ class Stocastic():
         self.distributions = {}
 
     def __eq__(self,other):
-        if isinstance(other,Stocastic):
+        if isinstance(other,Stochastic):
             if self.get_attributes() == other.get_attributes() and \
                 self.distributions == other.distributions:
                return True
         return False
+
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of Stochastic
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A Stochastic element (same as generated by the class itself)
+
+            Returns
+            -------
+                Stochastic (Stochastic): a Stochastic object
+                
+        """
+        
+        if 'randomSeed' in element.attrib:
+            seed = element.attrib['randomSeed']
+        else:
+            seed = None
+        stoc = Stochastic(element.attrib['numberOfTestRuns'], seed)
+        for dist_element in element.findall('StochasticDistribution'):
+            name = dist_element.attrib['parameterName']
+            dist = _StochasticFactory.parse_distribution(dist_element)
+            stoc.add_distribution(name,dist)
+        return stoc
+
     def add_distribution(self,parametername,distribution):
         """ adds a parameter and a related distribution to it
 
@@ -254,7 +376,11 @@ class Stocastic():
         """ returns the attributes of the Stochastic as a dict
 
         """
-        retdict = {'numberOfTestRuns':str(self.number_of_tests),'randomSeed':str(self.random_seed)}
+        retdict = {} 
+        retdict['numberOfTestRuns'] = str(self.number_of_tests)
+        if self.random_seed is not None:
+            retdict['randomSeed']= str(self.random_seed)
+
         return retdict
 
     def get_element(self):
@@ -272,7 +398,7 @@ class Stocastic():
         return element
 
 class NormalDistribution(_StochasticDistributionType):
-    """ the NormalDistribution is a stocastic distribution
+    """ the NormalDistribution is a Stochastic distribution
         
         Parameters
         ----------
@@ -293,6 +419,9 @@ class NormalDistribution(_StochasticDistributionType):
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -326,6 +455,26 @@ class NormalDistribution(_StochasticDistributionType):
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of NormalDistribution
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A NormalDistribution element (same as generated by the class itself)
+
+            Returns
+            -------
+                nd (NormalDistribution): a NormalDistribution object
+                
+        """
+        if element.find('Range') is not None:
+            range = Range.parse(element.find('Range'))
+        else:
+            range = None
+        nd = NormalDistribution(element.attrib['expectedValue'],element.attrib['variance'],range)
+        return nd
+
     def get_attributes(self):
         """ returns the attributes of the NormalDistribution as a dict
 
@@ -343,7 +492,7 @@ class NormalDistribution(_StochasticDistributionType):
         return element
 
 class UniformDistribution(_StochasticDistributionType):
-    """ the UniformDistribution is a stocastic distribution
+    """ the UniformDistribution is a Stochastic distribution
         
         Parameters
         ----------
@@ -355,11 +504,14 @@ class UniformDistribution(_StochasticDistributionType):
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
     """
-    def __init__(self, expected_value, variance, range = None):
+    def __init__(self, range):
         """ initalizes the UniformDistribution
 
         Parameters
@@ -368,7 +520,7 @@ class UniformDistribution(_StochasticDistributionType):
 
 
         """ 
-        if range and not isinstance(range,Range):
+        if not isinstance(range,Range):
             raise TypeError('range input is not of type Range')
         self.range = range
 
@@ -378,16 +530,32 @@ class UniformDistribution(_StochasticDistributionType):
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of UniformDistribution
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A UniformDistribution element (same as generated by the class itself)
+
+            Returns
+            -------
+                ud (UniformDistribution): a UniformDistribution object
+                
+        """
+        
+        return UniformDistribution(Range.parse(element.find('Range')))
+
     def get_element(self):
         """ returns the elementTree of the UniformDistribution
 
         """
-        element = ET.Element('UniformDistribution',self.get_attributes())
+        element = ET.Element('UniformDistribution')
         element.append(self.range.get_element())
         return element
 
 class PoissonDistribution(_StochasticDistributionType):
-    """ the PoissonDistribution is a stocastic distribution
+    """ the PoissonDistribution is a Stochastic distribution
         
         Parameters
         ----------
@@ -404,6 +572,9 @@ class PoissonDistribution(_StochasticDistributionType):
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -434,6 +605,26 @@ class PoissonDistribution(_StochasticDistributionType):
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of PoissonDistribution
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A PoissonDistribution element (same as generated by the class itself)
+
+            Returns
+            -------
+                pd (PoissonDistribution): a PoissonDistribution object
+                
+        """
+        if element.find('Range') is not None:
+            range = Range.parse(element.find('Range'))
+        else:
+            range = None
+        pd = PoissonDistribution(element.attrib['expectedValue'],range)
+        return pd
+
     def get_attributes(self):
         """ returns the attributes of the PoissonDistribution as a dict
 
@@ -451,7 +642,7 @@ class PoissonDistribution(_StochasticDistributionType):
         return element
 
 class Histogram(_StochasticDistributionType):
-    """ the Histogram is a stocastic distribution
+    """ the Histogram is a Stochastic distribution
         
         Parameters
         ----------
@@ -462,6 +653,9 @@ class Histogram(_StochasticDistributionType):
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -481,6 +675,24 @@ class Histogram(_StochasticDistributionType):
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of Histogram
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A Histogram element (same as generated by the class itself)
+
+            Returns
+            -------
+                histogram (Histogram): a Histogram object
+                
+        """
+        hist = Histogram()
+        for h_element in element.findall('HistogramBin'):
+            hist.bins.append(_HistogramBin.parse(h_element))
+        return hist
+
     def add_bin(self,weight,range):
         """ returns the attributes of the Histogram as a dict
 
@@ -506,7 +718,7 @@ class Histogram(_StochasticDistributionType):
         return element
 
 class ProbabilityDistributionSet(_StochasticDistributionType):
-    """ the ProbabilityDistributionSet is a stocastic distribution
+    """ the ProbabilityDistributionSet is a Stochastic distribution
         
         Parameters
         ----------
@@ -517,6 +729,9 @@ class ProbabilityDistributionSet(_StochasticDistributionType):
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class   
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -536,6 +751,24 @@ class ProbabilityDistributionSet(_StochasticDistributionType):
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of ProbabilityDistributionSet
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A ProbabilityDistributionSet element (same as generated by the class itself)
+
+            Returns
+            -------
+                pds (ProbabilityDistributionSet): a ProbabilityDistributionSet object
+                
+        """
+        pds = ProbabilityDistributionSet()
+        for e in element.findall('Element'):
+            pds.sets.append(_ProbabilityDistributionSetElement.parse(e))
+        return pds
+
     def add_set(self, value, weight):
         """ adds a set element to the ProbabilityDistributionSet
 
@@ -578,6 +811,9 @@ class ParameterValueSet():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             add_parameter(parameterref, value)
                 Adds a parameter value pair
 
@@ -598,6 +834,25 @@ class ParameterValueSet():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of ParameterValueSet
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A ParameterValueSet element (same as generated by the class itself)
+
+            Returns
+            -------
+                parameter_value_set (ParameterValueSet): a ParameterValueSet object
+                
+        """
+        parameter_value_set = ParameterValueSet()
+        for pa in element.findall('ParameterAssignment'):
+            parameter_value_set.sets.append(ParameterAssignment.parse(pa))
+        
+        return parameter_value_set
+
     def add_parameter(self,parameterref,value):
         """ adds a parameter and a value
 
@@ -639,6 +894,9 @@ class DeterministicMultiParameterDistribution():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             add_value_set(parameter_value_set)
                 Adds a parameter value set
 
@@ -659,6 +917,24 @@ class DeterministicMultiParameterDistribution():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of DeterministicMultiParameterDistribution
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A DeterministicMultiParameterDistribution element (same as generated by the class itself)
+
+            Returns
+            -------
+                distribution (DeterministicMultiParameterDistribution): a DeterministicMultiParameterDistribution object
+                
+        """
+        dist = DeterministicMultiParameterDistribution()
+        for h_element in element.findall('ValueSetDistribution/ParameterValueSet'):
+            dist.sets.append(ParameterValueSet.parse(h_element))
+        return dist
+
     def add_value_set(self,parameter_value_set):
         """ adds a parameter and a related distribution to it
 
@@ -690,20 +966,23 @@ class DistributionRange():
         
         Parameters
         ----------
-            step_width (double): step size of the distribution
+            step_width (float): step size of the distribution
 
             range (Range): the range of the parameter
 
         Attributes
         ----------
 
-            step_width (double): step size of the distribution
+            step_width (float): step size of the distribution
 
             range (Range): the range of the parameter
 
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -716,7 +995,7 @@ class DistributionRange():
 
             Parameters
             ----------
-                step_width (double): step size of the distribution
+                step_width (float): step size of the distribution
 
                 range (Range): the range of the parameter
 
@@ -734,6 +1013,23 @@ class DistributionRange():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of DistributionRange
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A DistributionRange element (same as generated by the class itself)
+
+            Returns
+            -------
+                distribution_range (DistributionRange): a DistributionRange object
+                
+        """
+        
+        distribution_range = DistributionRange(element.attrib['stepWidth'],Range.parse(element.find('Range')))
+        return distribution_range
+
     def get_attributes(self):
         """ returns the attributes of the DistributionRange as a dict
 
@@ -765,6 +1061,9 @@ class DistributionSet():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             get_element(elementname)
                 Returns the full ElementTree of the class
 
@@ -781,6 +1080,24 @@ class DistributionSet():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of DistributionSet
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A DistributionSet element (same as generated by the class itself)
+
+            Returns
+            -------
+                dist (DistributionSet): a DistributionSet object
+                
+        """
+        dist = DistributionSet()
+        for v in element.findall('Element'):
+            dist.add_value(v.attrib['value'])
+        return dist
+
     def add_value(self,value):
         """ adds a value to the DistirbutionSet
 
@@ -818,6 +1135,9 @@ class Deterministic():
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             add_multi_distribution(distribution)
                 Adds a DeterministicMultiParameterDistribution
 
@@ -842,6 +1162,29 @@ class Deterministic():
                return True
         return False
 
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of Deterministic
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A Deterministic element (same as generated by the class itself)
+
+            Returns
+            -------
+                deterministic (Deterministic): a Deterministic object
+                
+        """
+        deterministic = Deterministic()
+        for single_dist in element.findall('DeterministicSingleParameterDistribution'):
+            param_name = single_dist.attrib['parameterName']
+            dist = _DeterministicFactory.parse_distribution(single_dist)
+            deterministic.add_single_distribution(param_name,dist)
+        for multi_dist in element.findall('DeterministicMultiParameterDistribution'):
+            deterministic.add_multi_distribution(DeterministicMultiParameterDistribution.parse(multi_dist))
+
+        return deterministic
+
     def add_multi_distribution(self,distribution):
         """ adds a DeterministicMultiParameterDistribution
 
@@ -906,6 +1249,9 @@ class ParameterValueDistribution(VersionBase):
 
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             write_xml(filename)
                 write a open scenario xml
 
@@ -916,7 +1262,7 @@ class ParameterValueDistribution(VersionBase):
     _XMLNS = XMLNS
     _XSI = XSI
     def __init__(self, description, author, scenario_file, parameter_distribution, osc_minor_version=1):
-        """ initalizes the Stocastic 
+        """ initalizes the ParameterValueDistribution 
 
         Parameters
         ----------
@@ -926,14 +1272,14 @@ class ParameterValueDistribution(VersionBase):
 
             scenario_file (str): path to the scenario that the parameter distribution should set
 
-            parameter_distribution (Stocastic or Deterministic): the parameter distribution
+            parameter_distribution (Stochastic or Deterministic): the parameter distribution
 
             osc_minor_version (int): the osi version wanted (Note: only available from OpenSCENARIO V1.1.0)
 
         """ 
-        self.header = FileHeader(description,author,revMinor=osc_minor_version)
-        if not isinstance(parameter_distribution,Stocastic):
-            raise TypeError(type(parameter_distribution) + "is not of type Stocastic or Deterministic")
+        self.header = FileHeader(author,description,revMinor=osc_minor_version)
+        if not isinstance(parameter_distribution,Stochastic) and not isinstance(parameter_distribution,Deterministic):
+            raise TypeError(type(parameter_distribution) + "is not of type Stochastic or Deterministic")
         self.scenario_file = scenario_file
         self.parameter_distribution = parameter_distribution
 
@@ -945,8 +1291,35 @@ class ParameterValueDistribution(VersionBase):
                return True
         return False
  
+    @staticmethod
+    def parse(element):
+        """ Parses the xml element of ParameterValueDistribution
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A ParameterValueDistribution element (same as generated by the class itself)
+
+            Returns
+            -------
+                paramvaluedist (ParameterValueDistribution): a ParameterValueDistribution object
+                
+        """
+        
+        
+        pvd_element = element.find('ParameterValueDistribution')
+        if pvd_element.find('Stochastic') is not None:
+            dist = Stochastic.parse(pvd_element.find('Stochastic'))
+        elif pvd_element.find('Deterministic') is not None:
+            dist = Deterministic.parse(pvd_element.find('Deterministic'))
+        else:
+            raise NotAValidElement('ParameterValueDistribution is missing a distribution')
+        
+        pvd = ParameterValueDistribution('','',pvd_element.find('ScenarioFile').attrib['filepath'],dist)
+        pvd.header = FileHeader.parse(element.find('FileHeader'))
+        return pvd
+
     def get_element(self):
-        """ returns the elementTree of the Stochastic
+        """ returns the elementTree of the ParameterValueDistribution
 
         """
         if self.isVersion(minor=0):
@@ -955,6 +1328,7 @@ class ParameterValueDistribution(VersionBase):
         element.append(self.header.get_element())
         parameterdist = ET.SubElement(element,'ParameterValueDistribution')
         ET.SubElement(parameterdist,'ScenarioFile',attrib={'filepath':self.scenario_file})
+
         parameterdist.append(self.parameter_distribution.get_element())
             
         return element
@@ -971,4 +1345,6 @@ class ParameterValueDistribution(VersionBase):
                 Default: True
 
         """
-        printToFile(self.get_element(),filename,prettyprint)
\ No newline at end of file
+        printToFile(self.get_element(),filename,prettyprint)
+
+
